package ufrn.imd.controller.impl;

import lombok.extern.java.Log;
import ufrn.imd.controller.Controller;
import ufrn.imd.service.impl.AuthenticatorServiceImpl;
import ufrn.imd.utils.ServerResponse;

import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;

import static ufrn.imd.utils.NameSpace.HOST_AUTHENTICATOR_SERVICE;
import static ufrn.imd.utils.NameSpace.PORT_AUTHENTICATOR_SERVICE;

@Log
public class AuthenticatorControllerImpl extends UnicastRemoteObject implements Controller {
    private final AuthenticatorServiceImpl service;

    public AuthenticatorControllerImpl(AuthenticatorServiceImpl service) throws RemoteException, MalformedURLException, AlreadyBoundException {
        super();
        log.info("Starting Authenticator service!");
        this.service = service;
        log.info(String.format("Initializing server in %s", HOST_AUTHENTICATOR_SERVICE));
        LocateRegistry.createRegistry(PORT_AUTHENTICATOR_SERVICE);
        Naming.bind(HOST_AUTHENTICATOR_SERVICE, this);
    }

    @Override
    public ServerResponse getResponse(Object client) throws RemoteException {
        String accountNumber = (String) client;
        ServerResponse response;
        try {
            response = this.service.authenticate(accountNumber);
        } catch (RuntimeException e) {
            e.printStackTrace();
            response = ServerResponse.builder()
                    .status(500)
                    .message(e.getMessage())
                    .body(accountNumber)
                    .build();
        }
        return response;
    }
}
